@@ -53,23 +53,11 @@ extern "C" {
 
 #include "fcs_hash.h"
 
-#ifdef FCS_WITH_MHASH
-#include "mhash.h"
-#else
-#include "md5.h"
-#endif
-
 #endif
 
 #ifdef INDIRECT_STACK_STATES
 #include "fcs_hash.h"
 
-#ifdef FCS_WITH_MHASH
-#include "mhash.h"
-#else
-#include "md5.h"
-#endif
-
 #endif
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_DB_FILE)
@@ -80,6 +68,8 @@ extern "C" {
 
 #include "pqueue.h"
 
+#include "alloc.h"
+
 /*
  * This is a linked list item that is used to implement a queue for the BFS
  * scan.
@@ -123,6 +113,17 @@ enum FCS_TESTS_ORDER_FLAGS
 struct freecell_solver_hard_thread_struct;
 struct freecell_solver_soft_thread_struct;
 
+typedef struct freecell_solver_hard_thread_struct freecell_solver_hard_thread_t;
+
+struct fcs_tests_order_struct
+{
+    int num;
+    int * tests;
+    int max_num;
+};
+
+typedef struct fcs_tests_order_struct fcs_tests_order_t;
+
 typedef struct freecell_solver_instance
 {
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INDIRECT)
@@ -144,16 +145,6 @@ typedef struct freecell_solver_instance
     /* The number of states that were checked by the solving algorithm.
      * Badly named, should be renamed to num_iters or num_checked_states */
     int num_times;
-    /*
-     * A vector of the states leading to the solution. It is actively
-     * being modified by a Soft-DFS scan. Its usage for output has been
-     * deprecated because solution_moves was introduced.
-     * */
-    fcs_state_with_locations_t * * instance_solution_states;
-    /*
-     * The number of states in solution_states.
-     * */
-    int instance_num_solution_states;
 
     /*
      * A move stack that contains the moves leading to the solution.
@@ -162,11 +153,6 @@ typedef struct freecell_solver_instance
      * all the stacks of each depth.
      * */
     fcs_move_stack_t * solution_moves;
-    /*
-     * proto_solution_moves[i] are the moves that lead from
-     * solution_states[i] to solution_states[i+1].
-     * */
-    fcs_move_stack_t * * proto_solution_moves;
 
     /*
      * Limits for the maximal depth and for the maximal number of checked
@@ -233,11 +219,11 @@ typedef struct freecell_solver_instance
     SFO_hash_t * hash;
 #endif
 
+#if defined(INDIRECT_STACK_STATES)
     /*
      * The storage mechanism for the stacks assuming INDIRECT_STACK_STATES is
      * used.
-     * */
-#if defined(INDIRECT_STACK_STATES)
+     * */    
 #if (FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH)
     SFO_hash_t * stacks_hash;
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBAVL_AVL_TREE)
@@ -347,8 +333,7 @@ typedef struct freecell_solver_instance
      * This is the master tests order. It is used to initialize all
      * the new Soft-Threads.
      * */
-    int instance_tests_order_num;
-    int instance_tests_order[FCS_TESTS_NUM];
+    fcs_tests_order_t instance_tests_order;
 
     /*
      * This is the hard-thread used for the optimization scan.
@@ -361,8 +346,54 @@ typedef struct freecell_solver_instance
      * the instance terminates.
      * */
     int num_hard_threads_finished;
+
+    /*
+     * A flag that indicates whether or not to explicitly calculate
+     * the depth of a state that was reached.
+     * */
+    int calc_real_depth;
+
+    /*
+     * The tests order for the optimization scan as specified by the user.
+     * */
+    int opt_tests_order_set;
+    
+    fcs_tests_order_t opt_tests_order;
+
+    /*
+     * This flag indicates whether scans should or should not reparent the
+     * states their encounter to a lower depth in the depth tree
+     * */
+    int to_reparent_states;
+
+    /* 
+     * This variable determines how the scans cooperate with each other.
+     * 
+     * A value of 0 indicates that they don't and only share the same 
+     * states collection.
+     * 
+     * A value of 1 indicates that they mark states as dead-end,
+     * which may help or hinder other scans.
+     * */
+    int scans_synergy;
+
 } freecell_solver_instance_t;
 
+
+
+
+/***************************************************/
+
+
+struct fcs_prelude_item_struct
+{
+    int scan_idx;
+    int quota;
+};
+
+typedef struct fcs_prelude_item_struct fcs_prelude_item_t;
+
+
 struct freecell_solver_hard_thread_struct
 {
     freecell_solver_instance_t * instance;
@@ -423,15 +454,6 @@ struct freecell_solver_hard_thread_struct
      * access the instance.
      *
      * */
-#if (defined(INDIRECT_STACK_STATES) && (FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH)) || (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH)
-#ifdef FCS_WITH_MHASH
-    MHASH mhash_context;
-    hashid mhash_type;
-#else
-    MD5_CTX md5_context;
-    unsigned char hash_value[MD5_HASHBYTES];
-#endif
-#endif
 
     /*
      * The index for the soft-thread that is currently processed
@@ -444,9 +466,70 @@ struct freecell_solver_hard_thread_struct
      * this thread is skipped.
      * */
     int num_soft_threads_finished;
+
+#ifdef INDIRECT_STACK_STATES
+    /*
+     * This is the mechanism used to allocate memory for the stacks.
+     * */
+    fcs_compact_allocator_t * stacks_allocator;
+#endif
+
+    /* 
+     * This is a compact memory allocator for the move stacks associated
+     * with the states in the states collection.
+     * */
+    fcs_compact_allocator_t * move_stacks_allocator;
+
+    /* 
+     * This is a move stack that is used and re-used by the 
+     * tests functions of this hard thread 
+     * */
+    fcs_move_stack_t * reusable_move_stack;
+
+#ifdef INDIRECT_STACK_STATES
+    /*
+     * This is a buffer used to temporarily store the stacks of the duplicated
+     * state.
+     * */
+    char indirect_stacks_buffer[MAX_NUM_STACKS << 7];
+#else
+    char indirect_stacks_buffer[1];
+#endif
+
+    char * prelude_as_string;
+
+    int prelude_num_items;
+    int prelude_idx;
+    fcs_prelude_item_t * prelude;
+    
 };
 
-typedef struct freecell_solver_hard_thread_struct freecell_solver_hard_thread_t;
+
+
+
+
+/********************************************/
+
+
+
+
+
+
+
+
+struct fcs_soft_dfs_stack_item_struct
+{
+    fcs_state_with_locations_t * state;
+    fcs_derived_states_list_t derived_states_list;
+    int current_state_index;
+    int test_index;
+    int num_freestacks;
+    int num_freecells;
+    int derived_states_random_indexes_max_size;
+    int * derived_states_random_indexes;  
+};
+
+typedef struct fcs_soft_dfs_stack_item_struct fcs_soft_dfs_stack_item_t;
 
 struct freecell_solver_soft_thread_struct
 {
@@ -466,8 +549,8 @@ struct freecell_solver_soft_thread_struct
      * that it can be lower than FCS_TESTS_NUM, thus enabling several tests
      * to be removed completely.
      * */
-    int tests_order_num;
-    int tests_order[FCS_TESTS_NUM];
+    fcs_tests_order_t tests_order;
+
 
     /*
      * The (temporary) max depth of the Soft-DFS scans)
@@ -543,20 +626,8 @@ struct freecell_solver_soft_thread_struct
      * num_freecells[i] - ditto for the freecells.
      *
      * */
-    fcs_derived_states_list_t * soft_dfs_derived_states_lists;
-    int * soft_dfs_current_state_indexes;
-    int * soft_dfs_test_indexes;
-    int * soft_dfs_num_freestacks;
-    int * soft_dfs_num_freecells;
-    int * soft_dfs_derived_states_random_indexes_max_size;
-    int * * soft_dfs_derived_states_random_indexes;
-
-    /*
-     * A vector of the states leading to the solution. It is actively
-     * being modified by a Soft-DFS scan. Its usage for output has been
-     * deprecated because solution_moves was introduced.
-     * */
-    fcs_state_with_locations_t * * solution_states;
+    
+    fcs_soft_dfs_stack_item_t * soft_dfs_info;
 
     /* The depth of the DFS stacks */
     int num_solution_states;
@@ -566,6 +637,12 @@ struct freecell_solver_soft_thread_struct
      * */
     fcs_rand_t * rand_gen;
 
+    /*
+     * The initial seed of this random number generator
+     * */
+    int rand_seed;
+    
+
     /*
      * A flag that indicates if this soft thread have already been
      * initialized.
@@ -588,6 +665,11 @@ struct freecell_solver_soft_thread_struct
      * solve_instance() to skip to the next scan.
      * */
     int is_finished;
+
+    /*
+     * A malloced string that serves as an identification for the user.
+     * */
+    char * name;
 };
 
 typedef struct freecell_solver_soft_thread_struct freecell_solver_soft_thread_t;
@@ -696,6 +778,18 @@ extern int freecell_solver_a_star_or_bfs_solve(
 extern int freecell_solver_a_star_or_bfs_resume_solution(
     freecell_solver_soft_thread_t * soft_thread
     );
+
+extern int freecell_solver_soft_dfs_or_random_dfs_do_solve_or_resume(
+    freecell_solver_soft_thread_t * soft_thread,
+    fcs_state_with_locations_t * ptr_state_with_locations_orig,
+    int resume,
+    int to_randomize
+    );
+
+extern void freecell_solver_recycle_instance(
+    freecell_solver_instance_t * instance
+        );
+
 #ifdef __cplusplus
 }
 #endif
