@@ -36,52 +36,6 @@ fcs_card_t freecell_solver_empty_card = (fcs_card_t)0;
 
 #endif
 
-#ifdef INDIRECT_STACK_STATES
-void freecell_solver_clean_state(
-    fcs_state_with_locations_t * state
-    )
-{
-    int s;
-    for(s=0;s<MAX_NUM_STACKS;s++)
-    {
-        if (state->s.stacks[s] != NULL)
-        {
-            free(state->s.stacks[s]);
-        }
-    }
-}
-#endif
-#ifdef INDIRECT_STACK_STATES
-
-void freecell_solver_duplicate_state_proto(
-    fcs_state_with_locations_t * dest,
-    fcs_state_with_locations_t * src
-    )
-{
-    int a;
-    int stack_len;
-
-    *dest = *src;
-    for(a=0;a<MAX_NUM_STACKS;a++)
-    {
-        if (src->s.stacks[a] != NULL)
-        {
-            stack_len = src->s.stacks[a][0];
-#if 0
-            dest->s.stacks[a] = malloc(stack_len+14);
-            memset(dest->s.stacks[a], '\0', stack_len+14);
-#else
-            dest->s.stacks[a] = malloc(stack_len+53);
-            memset(dest->s.stacks[a], '\0', stack_len+53);
-#endif
-            memcpy(dest->s.stacks[a], src->s.stacks[a], stack_len+1);
-        }
-    }
-}
-
-#endif
-
-
 static int fcs_card_compare(const void * card1, const void * card2)
 {
     const fcs_card_t * c1 = (const fcs_card_t *)card1;
@@ -396,7 +350,13 @@ void freecell_solver_canonize_state(
 
 #endif
 
-static void fcs_state_init(fcs_state_with_locations_t * state, int stacks_num)
+static void fcs_state_init(
+    fcs_state_with_locations_t * state, 
+    int stacks_num
+#ifdef INDIRECT_STACK_STATES
+    ,char * indirect_stacks_buffer
+#endif
+    )
 {
     int a;
     memset((void*)&(state->s), 0, sizeof(fcs_state_t));
@@ -407,7 +367,7 @@ static void fcs_state_init(fcs_state_with_locations_t * state, int stacks_num)
 #ifdef INDIRECT_STACK_STATES
     for(a=0;a<stacks_num;a++)
     {
-        state->s.stacks[a] = malloc(MAX_NUM_DECKS*52+1);
+        state->s.stacks[a] = &indirect_stacks_buffer[a << 7];
         memset(state->s.stacks[a], '\0', MAX_NUM_DECKS*52+1);
     }
     for(;a<MAX_NUM_STACKS;a++)
@@ -470,6 +430,9 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
     int decks_num
 #ifdef FCS_WITH_TALONS
     ,int talon_type
+#endif
+#ifdef INDIRECT_STACK_STATES
+    , char * indirect_stacks_buffer
 #endif
     )
 {
@@ -485,7 +448,13 @@ fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
     int i;
     int decks_index[4];
 
-    fcs_state_init(&ret_with_locations, stacks_num);
+    fcs_state_init(
+        &ret_with_locations, 
+        stacks_num
+#ifdef INDIRECT_STACK_STATES        
+        , indirect_stacks_buffer  
+#endif
+        );
     str = string;
 
     first_line = 1;
@@ -900,11 +869,9 @@ char * freecell_solver_state_as_string(
     int a, card_num_is_null, b;
     int max_num_cards, s, card_num, len;
 
-    //char string[8000];
     char str2[128], str3[128], * str2_ptr, * str3_ptr;
 
     freecell_solver_append_string_t * app_str;
-    //char * str;
 
     int stack_locs[MAX_NUM_STACKS];
     int freecell_locs[MAX_NUM_FREECELLS];
@@ -947,7 +914,6 @@ char * freecell_solver_state_as_string(
             decks[a][0] = '0';
     }
 
-    //str = string;
     app_str = freecell_solver_append_string_alloc(512);
 
     if(!parseable_output)
