@@ -178,7 +178,7 @@ typedef int fcs_locs_t;
     ((state).talon[pos] = (card))
 #endif
 
-#define fcs_clean_state(state)
+#define fcs_copy_stack(state, idx, buffer) {} 
 
 #elif defined(COMPACT_STATES)    /* #ifdef DEBUG_STATES */
 
@@ -302,7 +302,7 @@ typedef char fcs_locs_t;
     fcs_push_card_into_stack((state), (ds), fcs_stack_card((state), (ss), (sc)))
 
 #define fcs_duplicate_state(dest, src) \
-    (dest) = (src);
+    (dest) = (src)
 
 #define fcs_put_card_in_freecell(state, f, card) \
     (state).data[FCS_FREECELLS_OFFSET+(f)] = (card);
@@ -323,8 +323,6 @@ typedef char fcs_locs_t;
     ( (card) >> 6 )
 
 
-#define fcs_clean_state(state)
-
 #ifdef FCS_WITH_TALONS
 #define fcs_talon_len(state) \
     ((state).talon_params[0])
@@ -342,7 +340,11 @@ typedef char fcs_locs_t;
 #define fcs_flip_stack_card(state, s, c) \
     (fcs_card_set_flipped(fcs_stack_card((state),(s),(c)), ((fcs_card_t)0) ))
 
-#elif defined(INDIRECT_STACK_STATES)
+#define fcs_copy_stack(state, idx, buffer) {} 
+    
+#elif defined(INDIRECT_STACK_STATES) /* #ifdef DEBUG_STATES
+                                        #elif defined(COMPACT_STATES)
+                                      */
 
 typedef char fcs_card_t;
 
@@ -451,12 +453,25 @@ typedef struct fcs_struct_state_t fcs_state_t;
     (fcs_card_set_flipped(fcs_stack_card(state,s,c), ((fcs_card_t)0) ))
 
 
-#define fcs_duplicate_state(dest, src) \
-    freecell_solver_duplicate_state_proto(&(dest), &(src));
-
-
-#define fcs_clean_state(state) freecell_solver_clean_state(state);
+#define fcs_duplicate_state(dest,src)           \
+    {                                           \
+        (dest) = (src);                         \
+        (dest).stacks_copy_on_write_flags = 0;    \
+    }
 
+#define fcs_copy_stack(state, idx, buffer) \
+    {     \
+        if (! ((state).stacks_copy_on_write_flags & (1 << idx)))        \
+        {          \
+            int stack_len;      \
+            (state).stacks_copy_on_write_flags |= (1 << idx);       \
+            stack_len = fcs_stack_len((state).s,idx);     \
+            memcpy(&buffer[idx << 7], (state).s.stacks[idx], stack_len+1); \
+            (state).s.stacks[idx] = &buffer[idx << 7];     \
+        }     \
+    }
+            
+            
 typedef char fcs_locs_t;
 
 #endif /* #ifdef DEBUG_STATES -
@@ -503,6 +518,12 @@ struct fcs_struct_state_with_locations_t
      * its scan has already visited this state
      * */
     int scan_visited[MAX_NUM_SCANS_BUCKETS];
+#ifdef INDIRECT_STACK_STATES
+    /* 
+     * A vector of flags that indicates which stacks were already copied.
+     * */
+    int stacks_copy_on_write_flags; 
+#endif
 };
 
 typedef struct fcs_struct_state_with_locations_t fcs_state_with_locations_t;
@@ -511,18 +532,6 @@ typedef struct fcs_struct_state_with_locations_t fcs_state_with_locations_t;
 extern fcs_card_t freecell_solver_empty_card;
 #define fcs_empty_card freecell_solver_empty_card
 
-#ifdef INDIRECT_STACK_STATES
-extern void freecell_solver_duplicate_state_proto(
-    fcs_state_with_locations_t * dest,
-    fcs_state_with_locations_t * src
-    );
-
-extern void freecell_solver_clean_state(
-    fcs_state_with_locations_t * state
-    );
-
-
-#endif
 
 #ifdef FCS_WITH_TALONS
 #define fcs_klondike_talon_len(state) \
@@ -590,6 +599,9 @@ extern fcs_state_with_locations_t freecell_solver_initial_user_state_to_c(
     int decks_num
 #ifdef FCS_WITH_TALONS
     ,int talon_type
+#endif
+#ifdef INDIRECT_STACK_STATES
+    , char * indirect_stacks_buffer
 #endif
     );
 
@@ -624,6 +636,7 @@ enum FCS_VISITED_T
     FCS_VISITED_IN_SOLUTION_PATH = 0x2,
     FCS_VISITED_IN_OPTIMIZED_PATH = 0x4,
     FCS_VISITED_DEAD_END = 0x8,
+    FCS_VISITED_ALL_TESTS_DONE = 0x10,
 };
 
 
